#!/usr/bin/env ruby

require 'optparse'
require 'erb'

TBLINUX_RELEASE_FILE        = '/etc/tblinux-release'
TBPRODUCT_EXEC              = '/usr/bin/tbproduct'
TBMAC_ADDR_FILE             = '/lib/tb/.tbmac_addr'
VLAN_CFG_LINE               = ERB.new <<-EOS
vlan_id=<%= vlan_cfg[:vlan_id] %>;mac_address=<%= vlan_cfg[:mac_addr] %><% unless vlan_cfg[:dhcp] %>;ip_address=<%= vlan_cfg[:ip_addr] %>;netmask=<%= vlan_cfg[:netmask] %>;ctrl=<%= vlan_cfg[:is_ctrl] ? "true" : "false" %><% end %>
EOS
PHYS_DEV_LINE               = ERB.new <<-EOS
device=<%= if_cfg[:device] %>;mac_address=<%= if_cfg[:mac_addr] %><% unless if_cfg[:dhcp] %>;ip_address=<%= if_cfg[:ip_addr] %>;netmask=<%= if_cfg[:netmask] %>;ctrl=<% end %>
EOS

IFCFG_HWADDR                = /HWaddr ([0-9a-fA-F]{2}:[0-9a-fA-F]{2}:[0-9a-fA-F]{2}:[0-9a-fA-F]{2}:[0-9a-fA-F]{2}:[0-9a-fA-F]{2})/
MAC_BYTE_DOT		    = /([0-9a-fA-F]{2}):([0-9a-fA-F]{2}):([0-9a-fA-F]{2}):([0-9a-fA-F]{2}):([0-9a-fA-F]{2}):([0-9a-fA-F]{2})/
MAC_BYTE_DASH		    = /([0-9a-fA-F]{2})-([0-9a-fA-F]{2})-([0-9a-fA-F]{2})-([0-9a-fA-F]{2})-([0-9a-fA-F]{2})-([0-9a-fA-F]{2})/

RHEL_NETWORK_SCRIPTS_PATH                = '/etc/sysconfig/network-scripts/'
RHEL_NETWORK_MGMT_IFUP_EXEC              = '/sbin/ifup'
RHEL_NETWORK_MGMT_IFDOWN_EXEC            = '/sbin/ifdown'
RHEL_NETWORK_MGMT_IFUPDOWN_DELAY         = 1
RHEL_NETWORK_MGMT_UPDATE_NEIGHBOURS_EXEC = '/sbin/arping'
RHEL_NETWORK_SCRIPT_MAGIC                = '# vlan_management.rb'
RHEL_NETWORK_SCRIPT_CTRL_MAGIC           = '# ctrl vlan_management.rb'

RHEL_NETWORK_SCRIPT_IS_VLAN   = /VLAN=(\w+)/
RHEL_NETWORK_SCRIPT_DEVICE    = /DEVICE=(\w+)/
RHEL_NETWORK_SCRIPT_VLAN_ID   = /vlan([\d]{1,3})/
RHEL_NETWORK_SCRIPT_IS_BOND   = /bond([\d]{1,3})/
RHEL_NETWORK_SCRIPT_MASTER    = /MASTER=(\w+)/
RHEL_NETWORK_SCRIPT_IS_SLAVE  = /SLAVE=(\w+)/
RHEL_NETWORK_SCRIPT_PHYSDEV   = /PHYSDEV=(\w+)/
RHEL_NETWORK_SCRIPT_BOOTPROTO = /BOOTPROTO=(\w+)/
RHEL_NETWORK_SCRIPT_IPADDR    = /IPADDR=(\d{1,3}.\d{1,3}.\d{1,3}.\d{1,3})/
RHEL_NETWORK_SCRIPT_NETMASK   = /NETMASK=(\d{1,3}.\d{1,3}.\d{1,3}.\d{1,3})/
RHEL_NETWORK_SCRIPT_MACADDR   = /MACADDR=([0-9a-fA-F]{2}:[0-9a-fA-F]{2}:[0-9a-fA-F]{2}:[0-9a-fA-F]{2}:[0-9a-fA-F]{2}:[0-9a-fA-F]{2})/
RHEL_NETWORK_SCRIPT_HWADDR    = /HWADDR=([0-9a-fA-F]{2}:[0-9a-fA-F]{2}:[0-9a-fA-F]{2}:[0-9a-fA-F]{2}:[0-9a-fA-F]{2}:[0-9a-fA-F]{2})/
RHEL_NETWORK_SCRIPT_DHCP_HOSTNAME = /DHCP_HOSTNAME=(\w+)/
RHEL_NETWORK_SCRIPT_MAGIC_RE  = /#{RHEL_NETWORK_SCRIPT_MAGIC}/
RHEL_NETWORK_SCRIPT_CTRL_MAGIC_RE  = /#{RHEL_NETWORK_SCRIPT_CTRL_MAGIC}/

RHEL_NETWORK_SCRIPT_VLAN_TEMPLATE = ERB.new <<-EOS
<% if vlan_cfg[:is_ctrl] %>
<%= RHEL_NETWORK_SCRIPT_CTRL_MAGIC %>
<% elsif vlan_cfg[:is_managed] %>
<%= RHEL_NETWORK_SCRIPT_MAGIC %>
<% end %>
VLAN=yes
VLAN_NAME_TYPE=VLAN_PLUS_VID_NO_PAD
DEVICE=vlan<%= vlan_cfg[:vlan_id] %>
PHYSDEV=<%= vlan_cfg[:int_iface] %>
BOOTPROTO=<%= vlan_cfg[:dhcp] ? 'dhcp' : 'static' %>
ONBOOT=yes
TYPE=Ethernet
<% unless vlan_cfg[:dhcp] %>
IPADDR=<%= vlan_cfg[:ip_addr] %>
NETMASK=<%= vlan_cfg[:netmask] %>
<% end %>
MACADDR=<%= vlan_cfg[:mac_addr] %>
EOS

RHEL_NETWORK_SCRIPT_PHYS_DEV_TEMPLATE = ERB.new <<-EOS
<%= RHEL_NETWORK_SCRIPT_CTRL_MAGIC %>
DEVICE=<%= if_cfg[:device] %>
BOOTPROTO=<%= if_cfg[:dhcp] ? 'dhcp' : 'static' %>
DHCPCLASS=
HWADDR=<%= if_cfg[:mac_addr] %>
ONBOOT=yes
DHCP_HOSTNAME=<%= if_cfg[:dhcp_host_name] %>
<% unless if_cfg[:dhcp] %>
IPADDR=<%= if_cfg[:ip_addr] %>
NETMASK=<%= if_cfg[:netmask] %>
<% end %>
EOS

RHEL_NETWORK_SCRIPT_BOND_PHYS_DEV_TEMPLATE = ERB.new <<-EOS
<% if vlan_cfg[:is_ctrl] %>
<%= RHEL_NETWORK_SCRIPT_CTRL_MAGIC %>
<% elsif vlan_cfg[:is_managed] %>
<%= RHEL_NETWORK_SCRIPT_MAGIC %>
<% end %>
DEVICE=<%= phys_dev_cfg[:device] %>
USERCTL=no
ONBOOT=yes
MASTER=bond0
SLAVE=yes
BOOTPROTO=none

EOS

RHEL_NETWORK_SCRIPT_BOND_TEMPLATE = ERB.new <<-EOS
<% if vlan_cfg[:is_ctrl] %>
<%= RHEL_NETWORK_SCRIPT_CTRL_MAGIC %>
<% elsif vlan_cfg[:is_managed] %>
<%= RHEL_NETWORK_SCRIPT_MAGIC %>
<% end %>
DEVICE=<%= bond_cfg[:device] %>
ONBOOT=yes
BOOTPROTO=<%= bond_cfg[:dhcp] ? 'dhcp' : 'static' %>
<% unless bond_cfg[:dhcp] %>
IPADDR=<%= bond_cfg[:ip_addr] %>
NETMASK=<%= bond_cfg[:netmask] %>
<% end %>
USERCTL=no
BONDING_OPTS="mode=active-backup miimon=100"

EOS

def print_result str
  if $result_file
    $result_file.puts str
  else
    puts "[OK]\t #{str}"
  end
end

def get_platform
  if File.exists? TBLINUX_RELEASE_FILE
    return :tmg_tblinux
  end
  
  if File.exists? TBPRODUCT_EXEC
    IO.popen TBPRODUCT_EXEC do |io|
      line = io.readline
      case line.strip
      when 'TMG800'
        return :tmg_800
      when 'TMG3200'
        return :tmg_3200
      when 'TMG7800-CTRL'
        return :tmg_7800_ctrl
      end
    end
  end
  
  return :generic
end

def get_int_iface
  case $platform
  when :tmg_tblinux
    return 'eth1'
  when :tmg_800
    return 'mgmt0'
  when :tmg_3200
    return 'int0'
  when :tmg_7800_ctrl
    # TODO: Bonding
  else
    return 'dummy0'
  end
  
  return nil
end

def get_iface_mac( iface )
  begin
    IO.popen "ifconfig #{iface}" do |io|
      if io.read =~ IFCFG_HWADDR
        $1 =~ MAC_BYTE_DOT
        return "#{$1}-#{$2}-#{$3}-#{$4}-#{$5}-#{$6}"
      end
    end
  rescue
  end
end

def get_excluded_vlans
  case $platform
  when :tmg_800
    { 3 => true }
  else
    {}
  end
end

def get_network_interface_list
  interface_list = []

  Dir.glob(get_network_script_path + 'ifcfg-*') do |ifcfg_file|
    File.open(ifcfg_file, 'r') do |vlan_cfg|
      interface = {}
      vlan_cfg.readlines.each do |line|
        if line =~ RHEL_NETWORK_SCRIPT_IS_VLAN
          interface[:is_vlan] = true
        elsif line =~ RHEL_NETWORK_SCRIPT_DEVICE
          device = $1
          interface[:device] = device
          if device =~ RHEL_NETWORK_SCRIPT_VLAN_ID
            interface[:vlan_id] = $1.to_i
          elsif device =~ RHEL_NETWORK_SCRIPT_IS_BOND
            interface[:is_bond] = true
          end
        elsif line =~ RHEL_NETWORK_SCRIPT_MASTER
          interface[:master] = $1
        elsif line =~ RHEL_NETWORK_SCRIPT_IS_SLAVE
          interface[:is_slave] = true
        elsif line =~ RHEL_NETWORK_SCRIPT_PHYSDEV
          interface[:int_iface] = $1
        elsif line =~ RHEL_NETWORK_SCRIPT_BOOTPROTO
          if $1 == 'dhcp'
            interface[:dhcp] = true
          end
        elsif line =~ RHEL_NETWORK_SCRIPT_IPADDR
          interface[:ip_addr] = $1
        elsif line =~ RHEL_NETWORK_SCRIPT_NETMASK
          interface[:netmask] = $1
        elsif line =~ RHEL_NETWORK_SCRIPT_MACADDR
       	  $1 =~ MAC_BYTE_DOT
	        interface[:mac_addr] = "#{$1}-#{$2}-#{$3}-#{$4}-#{$5}-#{$6}"
        elsif line =~ RHEL_NETWORK_SCRIPT_HWADDR
       	  $1 =~ MAC_BYTE_DOT
	        interface[:mac_addr] = "#{$1}-#{$2}-#{$3}-#{$4}-#{$5}-#{$6}"
        elsif line =~ RHEL_NETWORK_SCRIPT_DHCP_HOSTNAME
          interface[:dhcp_host_name] = $1
        elsif line =~ RHEL_NETWORK_SCRIPT_MAGIC_RE
          interface[:is_managed] = true
        elsif line =~ RHEL_NETWORK_SCRIPT_CTRL_MAGIC_RE
          interface[:is_ctrl] = true
        end
      end
      
      interface_list << interface
    end
  end
  
  interface_list
end

def get_vlan_list
  vlan_list = {}
  
  get_network_interface_list.each do |vlan_cfg|
    # Only list vlans on the internal interface
    if vlan_cfg[:is_vlan] and vlan_cfg[:int_iface] == get_int_iface
      vlan_list[vlan_cfg[:vlan_id]] = vlan_cfg
    end
  end
  
  vlan_list
end

def get_phys_dev_list
  phys_dev_list = []
  
  get_network_interface_list.each do |phys_dev_cfg|
    next if phys_dev_cfg[:is_vlan] or phys_dev_cfg[:is_bond]
    next if phys_dev_cfg[:device] == 'lo'
    
    phys_dev_list << phys_dev_cfg
  end
  
  phys_dev_list
end

def get_bond_list
  bond_list = []
  
  get_network_interface_list.each do |bond_cfg|
    if bond_cfg[:is_bond]
      bond_list << bond_cfg
    end
  end
  
  bond_list
end

def get_available_mac
  int_iface_mac = get_iface_mac(get_int_iface)
  case $platform
  when :tmg_tblinux
    int_iface_mac
  else
    file_path = TBMAC_ADDR_FILE
    unless File.exist? file_path
      # try with a file in debug directory
      file_path = "./debug-network-scripts/tbmac_addr.txt"
      unless File.exist? file_path
        file_path = ".tbmac_addr"
        raise 'No available macs' unless int_iface_mac
        return int_iface_mac
      end
    end

    if $platform == :tmg_800
      excluded_mac = { int_iface_mac => true }
    else
      excluded_mac = {}
    end
    
    get_vlan_list.each do | vlan_id, vlan_cfg |
      excluded_mac[vlan_cfg[:mac_addr]] = true
    end
    
    File.open(file_path, 'r') do |io|
      io.readlines.each do |line|
       mac_addr = line.strip
        unless excluded_mac[mac_addr]
          return mac_addr
        end
      end
    end

    # No more available macs
    raise 'No available macs'
  end
end

def get_network_script_path
  case $platform
  when :tmg_tblinux, :tmg_3200
    return RHEL_NETWORK_SCRIPTS_PATH
  when :tmg_800
    return RHEL_NETWORK_SCRIPTS_PATH
  when :tmg_3200
    return RHEL_NETWORK_SCRIPTS_PATH
  when :tmg_7800_ctrl
    return RHEL_NETWORK_SCRIPTS_PATH
  end

  return "./debug-network-scripts/"
end

def get_if_up_script_path
  case $platform
  when :tmg_tblinux, :tmg_3200
    return RHEL_NETWORK_MGMT_IFUP_EXEC
  when :tmg_800
    return RHEL_NETWORK_MGMT_IFUP_EXEC
  when :tmg_3200
    return RHEL_NETWORK_MGMT_IFUP_EXEC
  when :tmg_7800_ctrl
    return RHEL_NETWORK_MGMT_IFUP_EXEC
  end

  return "ruby ./debug-network-scripts/ifup_simul.rb"
end

def get_if_down_script_path
  case $platform
  when :tmg_tblinux, :tmg_3200
    return RHEL_NETWORK_MGMT_IFDOWN_EXEC
  when :tmg_800
    return RHEL_NETWORK_MGMT_IFDOWN_EXEC
  when :tmg_3200
    return RHEL_NETWORK_MGMT_IFDOWN_EXEC
  when :tmg_7800_ctrl
    return RHEL_NETWORK_MGMT_IFDOWN_EXEC
  end

  return "ruby ./debug-network-scripts/ifdown_simul.rb"
end

def list options
  excluded_vlans = get_excluded_vlans
  get_vlan_list.each do |vlan_id, vlan_cfg|
    if excluded_vlans[vlan_id]
      next
    end
    if options[:is_managed] || options[:is_ctrl]
      # We want to list only managed and/or ctrl vlans
      fKeep = false
      if vlan_cfg[:is_managed] && options[:is_managed]
        fKeep = true
      end
      if vlan_cfg[:is_ctrl] && options[:is_ctrl]
        fKeep = true
      end
      next unless fKeep
    end
    print_result VLAN_CFG_LINE.result(binding)
  end
end

def list_physical options
  get_phys_dev_list.each do |if_cfg|
    print_result PHYS_DEV_LINE.result(binding)
  end
end

def destroy vlan_cfg, params = {}
  vlan_cfg = get_vlan_list[vlan_cfg[:vlan_id]]
  if vlan_cfg.nil?
    raise 'Specified vlan id does not exist'
  end
  
  system(get_if_down_script_path + " vlan#{vlan_cfg[:vlan_id]}")
  sleep RHEL_NETWORK_MGMT_IFUPDOWN_DELAY
  
  File.delete get_network_script_path + "ifcfg-vlan#{vlan_cfg[:vlan_id]}"
  
  print_result VLAN_CFG_LINE.result(binding) unless params[:quiet]
end

def configure vlan_cfg, params = {}
  vlan_cfg[:int_iface] = get_int_iface
  vlan_cfg[:mac_addr] = get_available_mac
  
  legacy_vlan = get_vlan_list[vlan_cfg[:vlan_id]]
  if legacy_vlan
    unless legacy_vlan[:is_managed] || legacy_vlan[:is_ctrl]
      # Legacy vlan found let's delete it and create the new one
      destroy legacy_vlan, :quiet => true
    else
      raise 'Specified vlan id already exists'
    end
  end

  #change the vlan format before writing it in the file
  if vlan_cfg[:mac_addr].include? '-'
    vlan_cfg[:mac_addr] =~ MAC_BYTE_DASH
    vlan_cfg[:mac_addr] = "#{$1}:#{$2}:#{$3}:#{$4}:#{$5}:#{$6}"
  end

  vlan_file_path = get_network_script_path + "ifcfg-vlan#{vlan_cfg[:vlan_id]}"
  cfg_file = File.open(vlan_file_path, 'w+')
  if cfg_file
    cfg_file.write RHEL_NETWORK_SCRIPT_VLAN_TEMPLATE.result(binding)
  else
    raise "Failed to create VLAN configuration file " + vlan_file_path 
  end
  cfg_file.close
  
  result = system(get_if_up_script_path + " vlan#{vlan_cfg[:vlan_id]}")

  unless result
    # Failed to restart networking. Destroy the VLAN file that we just created
    cfg_file = nil
    File.delete get_network_script_path + "ifcfg-vlan#{vlan_cfg[:vlan_id]}"
    raise "Shell command to create VLAN returned error " + $?.to_s + ". Command was: " + get_if_up_script_path + " vlan#{vlan_cfg[:vlan_id]}"
  end


  sleep RHEL_NETWORK_MGMT_IFUPDOWN_DELAY
  
  print_result VLAN_CFG_LINE.result(binding) unless params[:quiet]
end

def modify vlan_cfg, params = {}
  vlan_cfg[:int_iface] = get_int_iface
  vlan_cfg[:mac_addr] = get_available_mac
  
  if get_vlan_list[vlan_cfg[:vlan_id]].nil?
    raise 'Specified vlan id does not exist'
  end
    
  destroy vlan_cfg, :quiet => true
  configure vlan_cfg, :quiet => true
  
  print_result VLAN_CFG_LINE.result(binding) unless params[:quiet]
end

def modify_physical if_cfg_modif, params = {}

  # Search the physical interface to modify
  if_cfg = nil
  get_phys_dev_list.each do |if_cfg_itr|
    if if_cfg_itr[:device] == if_cfg_modif[:device]
      found = true

      # Build the parameters to use for the interface by
      #   - Starting with current interface we just read
      #   - Copy some parameters passed to this function that we allow changing:
      #     - IP address
      #     - Mask
      #     - DHCP
      if_cfg = if_cfg_itr
      if_cfg[:dhcp] = if_cfg_modif[:dhcp]
      if_cfg[:ip_addr] = if_cfg_modif[:ip_addr]
      if_cfg[:netmask] = if_cfg_modif[:netmask]

      break
    end
  end
  raise "Failed to find physical interface #{if_cfg[:device]} " unless if_cfg
  

  #change the vlan format before writing it in the file
  if if_cfg[:mac_addr].include? '-'
    if_cfg[:mac_addr] =~ MAC_BYTE_DASH
    if_cfg[:mac_addr] = "#{$1}:#{$2}:#{$3}:#{$4}:#{$5}:#{$6}"
  end

  vlan_file_path = get_network_script_path + "ifcfg-#{if_cfg[:device]}"
  cfg_file = File.open(vlan_file_path, 'w+')
  if cfg_file
    cfg_file.write RHEL_NETWORK_SCRIPT_PHYS_DEV_TEMPLATE.result(binding)
  else
    raise "Failed to create VLAN configuration file " + vlan_file_path 
  end
  cfg_file.close
  
  # Take the interface down, then up
  result = system(get_if_down_script_path + " #{if_cfg[:device]}")
  unless result
    # Failed to restart networking
    cfg_file = nil
    raise "Shell command to bring down interface returned error " + $?.to_s + ". Command was: " + get_if_down_script_path + " #{if_cfg[:device]}"
  else
    result = system(get_if_up_script_path + " #{if_cfg[:device]}")
    unless result
      # Failed to restart networking
      cfg_file = nil
      raise "Shell command to bring up interface returned error " + $?.to_s + ". Command was: " + get_if_up_script_path + " #{if_cfg[:device]}"
    else
      sleep RHEL_NETWORK_MGMT_IFUPDOWN_DELAY
    end
  end
  
  print_result PHYS_DEV_LINE.result(binding) unless params[:quiet]
end

def update_neighbours vlan_cfg, params = {}
  vlan_cfg = get_vlan_list[vlan_cfg[:vlan_id]]
  if vlan_cfg.nil?
    raise 'Specified vlan id does not exist'
  end
  
  system(RHEL_NETWORK_MGMT_UPDATE_NEIGHBOURS_EXEC + " -q -A -c 1 -I vlan#{vlan_cfg[:vlan_id]} #{vlan_cfg[:ip_addr]}")
  
  print_result VLAN_CFG_LINE.result(binding) unless params[:quiet]
end

def configure_1_plus_1 vlan_cfg_ctrl0, vlan_cfg_ctrl1
  unless vlan_cfg_ctrl0
    raise 'Missing ctrl0 vlan config'
  end
  unless vlan_cfg_ctrl1
    raise 'Missing ctrl1 vlan config'
  end
	
  vlan_cfg_ctrl0[:int_iface] = get_int_iface
  vlan_cfg_ctrl0[:mac_addr] = get_available_mac
  vlan_cfg_ctrl0[:is_ctrl ] = true
  vlan_cfg_ctrl1[:int_iface] = get_int_iface
  vlan_cfg_ctrl1[:mac_addr] = get_available_mac
  vlan_cfg_ctrl1[:is_ctrl ] = true
  
  vlan_list = get_vlan_list
  if vlan_list[vlan_cfg_ctrl0[:vlan_id]] 
    raise 'Specified ctrl0 vlan id already exists'
  end
  if vlan_list[vlan_cfg_ctrl1[:vlan_id]]
    raise 'Specified ctrl1 vlan id already exists'
  end
	
  configure vlan_cfg_ctrl0
  configure vlan_cfg_ctrl1
end

options = {}
OptionParser.new do |opts|
  opts.banner = "Usage: #{File.basename($0)} [options]"
  
  opts.separator ""
  opts.separator "Specific options:"
  
  opts.on("--list", "List configured managed vlans") do |list|
    options[:action] = :list
    options[:is_managed] = true
  end
  
  opts.on("--list-ctrl", "List configured ctrl vlans") do |list|
    options[:action] = :list
    options[:is_ctrl] = true
  end
  
  opts.on("--list-physical", "List physcal IP interfaces") do |list|
    options[:action] = :list_physical
  end
  
  opts.on("--configure", "Configure a new vlan") do
    options[:action] = :configure
    options[:is_managed] = true
  end
  
  opts.on("--modify", "Modify an existing vlan") do
    options[:action] = :modify
    options[:is_managed] = true
  end
  
  opts.on("--modify-physical", "Modify the IP address of an existing physical IP interface") do
    options[:action] = :modify_physical
  end
  
  opts.on("--destroy", "Destroy an existing vlan") do
    options[:action] = :destroy
    options[:is_managed] = true
  end
  
  opts.on("--update-neighbours", "Tell neighbours that our MAC has changed port") do
    options[:action] = :update_neighbours
  end
  
  opts.on("--configure-1-plus-1", "Configure 1+1 vlans") do
    options[:action] = :configure_1_plus_1
    options[:is_ctrl] = true
  end
  
  opts.on("--result-file RESULT_FILE", "Set the result file PATH") do |result_file|
    options[:result_file] = result_file
  end
  
  opts.on("--device DEVICE", "Set the name of the physical device to apply the command to") do |device|
    options[:device] = device
  end
  
  opts.on("--vlan-id VLAN_ID", "Set the VLAN_ID") do |vlan_id|
    if options[:sub_vlan]
      options[options[:sub_vlan]] = {} unless options[options[:sub_vlan]]
      options[options[:sub_vlan]][:vlan_id] = vlan_id.to_i
    else
      options[:vlan_id] = vlan_id.to_i
    end
  end
  
  opts.on("--ip-address IP_ADDRESS", "Set the IP_ADDRESS") do |ip_address|
    if options[:sub_vlan]
      options[options[:sub_vlan]] = {} unless options[options[:sub_vlan]]
      options[options[:sub_vlan]][:ip_addr] = ip_address
    else
      options[:ip_addr] = ip_address
    end
  end
  
  opts.on("--netmask NETMASK", "Set the NETMASK") do |netmask|
    if options[:sub_vlan]
      options[options[:sub_vlan]] = {} unless options[options[:sub_vlan]]
      options[options[:sub_vlan]][:netmask] = netmask
    else
      options[:netmask] = netmask
    end
  end
  
  opts.on("--dhcp", "Indicates if vlan is to be auto-configured by DHCP") do |dhcp|
    if options[:sub_vlan]
      options[options[:sub_vlan]] = {} unless options[options[:sub_vlan]]
      options[options[:sub_vlan]][:dhcp] = dhcp
    else
      options[:dhcp] = dhcp
    end
  end
  
  opts.on("--ctrl-vlan", "Indicates that this is a VLAN used for ctrl") do
      options[:is_ctrl] = true
  end
  
  opts.on("--slave-interfaces INTERFACES", "Set the slave INTERFACES for bonding") do |interface|
      options[:slave_interfaces] = interface.split ','
  end
	
	opts.on("--ctrl0", "Further params are for ctrl0") do
    options[:sub_vlan] = :ctrl0
  end
	
	opts.on("--ctrl1", "Further params are for ctrl1") do
    options[:sub_vlan] = :ctrl1
  end
  
  opts.on_tail("-h", "--help", "Show this message") do
    puts opts
    exit
  end
end.parse!

unless options[:action]
  puts "[FAIL]\t Action argument is mandatory (use option --help to see a list of available actions)"
  Process.exit -1
end

$result_file = nil
if options[:result_file]
  $result_file = File.new( options[:result_file], 'w' )
end

# Code disabled: We don't support all platforms, but for non-supported applications
# we use debugging folder debug-network-scripts
$platform = get_platform
#case $platform 
#when :tmg_7800_ctrl, :generic
#  puts "[FAIL]\t Platform not supported"
#  Process.exit -1
#end

puts "[OK]\t Running vlan management script (#{options[:action].to_s}) on #{$platform.to_s}"

begin
  case options[:action]
  when :list
    list options
  when :list_physical
    list_physical options
  when :configure
    configure options
  when :modify
    modify options
  when :modify_physical
    modify_physical options
  when :destroy
    destroy options
  when :update_neighbours
    update_neighbours options
  when :configure_1_plus_1
    configure_1_plus_1 options[:ctrl0], options[:ctrl1]
  end
rescue
  puts "[FAIL]\t #{$!}"
  Process.exit -1
end

$result_file.close if $result_file
